// include the I2C library:
#include <Wire.h>

#define ADDR 8 // I2C address

// include the RFID reader library:
#include <PN532_I2C.h>
#include <PN532.h>
#include <NfcAdapter.h>

PN532_I2C pn532_i2c(Wire);
NfcAdapter nfc = NfcAdapter(pn532_i2c);

// include the SD library:
/*
  #include <SPI.h>
  #include <SD.h>
*/
#include "image.h"

// define the leds and buttons pin
#define LED_R_PIN 5 // pin for the RED LED
#define LED_Y_PIN 7 // pin for the YELLOW LED
#define LED_G_PIN 6 // pin for the GREEN LED
#define BTN_Y_PIN 8 // pin for the YES BUTTON
#define BTN_N_PIN 9 // pin for the NO BUTTON


#include "SoftwareSerial.h"
#define TX_PIN 4 // Arduino transmit  YELLOW WIRE  labeled RX on printer
#define RX_PIN 3 // Arduino receive   GREEN WIRE   labeled TX on printer

// include the thermal printer library
#include "Adafruit_Thermal.h"

SoftwareSerial mySerial(RX_PIN, TX_PIN); // Declare SoftwareSerial obj first
Adafruit_Thermal printer(&mySerial);     // Pass addr to printer constructor

// Tags name
#define ADMIN_TYPE "text/admin"  // content type of nfc tag
#define USERNAME "myUsername" // username of nfc tag

#define FILE_TYPE "text/print"  // content type of nfc tag
#define TUTORIAL_TYPE "text/tutorial"  // content type of nfc tag

String contentType = "";
String text = "";

void setup() {
  Serial.begin(115200); // initialize serial communication
  Serial.println("Start...");
  //Wire.begin(); // join i2c bus
  nfc.begin(); // initialize the NFC reader

  /*
    if (!SD.begin()) {
      Serial.println("initialization failed! Is a card inserted?");
    } else {
      Serial.println("initialization SD card done.");
    }
  */

  // Thermal printer begin
  mySerial.begin(19200); // Initialize SoftwareSerial
  printer.begin(); // Init printer (same regardless of serial type)
  printer.sleep(); // Tell printer to sleep

  // Leds begin
  pinMode(LED_R_PIN, OUTPUT);      // make the led pin an output
  pinMode(LED_Y_PIN, OUTPUT);      // make the led pin an output
  pinMode(LED_G_PIN, OUTPUT);      // make the led pin an output
  pinMode(BTN_Y_PIN, INPUT_PULLUP);      // make the led pin an output
  pinMode(BTN_N_PIN, INPUT_PULLUP);      // make the led pin an output

}

void loop() {
  if (nfc.tagPresent()) {
    readTag();
    Serial.print("Content type:\t");
    Serial.println(contentType);
    Serial.print("text:\t");
    Serial.println(text);
    Serial.println();

    if (contentType == ADMIN_TYPE) {
      if (text == USERNAME) {
        Serial.println("lock");
        lock();
      } else {
        blink(LED_R_PIN);
      }
    } else if (contentType == TUTORIAL_TYPE) {
        blink(LED_Y_PIN);
      // Check to see if the file exists:
      if (thermalPrint(text)) {
        Serial.print(text);
        Serial.println(" printed.");
        blink(LED_Y_PIN);
      } else {
        Serial.print(text);
        Serial.println(" doesn't exist.");
        blink(LED_R_PIN);
      }


    }
  }

}

void readTag() {
  contentType = "";
  text = "";
  NfcTag tag = nfc.read();
  if (tag.hasNdefMessage()) {   // every tag won't have a message
    NdefMessage message = tag.getNdefMessage();
    NdefRecord record = message.getRecord(0);

    if (record.getTnf() == TNF_MIME_MEDIA) {

      contentType = record.getType(); // IMPORTANT
      // Get the length of the payload:
      int payloadLength = record.getPayloadLength();
      byte payload[payloadLength];  // make a byte array to hold the payload
      record.getPayload(payload);

      // convert the payload to a String
      for (int c = 0; c < payloadLength; c++) {
        if (isprint((char)payload[c])) {
          text += (char)payload[c];
        }
      }
    }
  }
}

boolean thermalPrint(String file) {
  printer.wake(); // MUST wake() before printing again, even if reset
  delay(1000); // Wait for 1 seconds
  printer.printBitmap(image_width, image_height, image_data);
  printer.println();
  printer.sleep(); // Tell printer to sleep
  return true;
  /*
    File file2print;

    if (SD.exists(file)) {
    file2print = SD.open(text, FILE_READ);
    printer.wake(); // MUST wake() before printing again, even if reset
    delay(1000); // Wait for 1 seconds
    while (file2print.available()) {
      printer.print(file2print.read());
    }
    file2print.close();

    printer.sleep(); // Tell printer to sleep
    return true;
    } else {
    return false;
    }*/
}

boolean lock() {
  Wire.requestFrom(ADDR, 1);    // request 1 byte from slave device #8
  bool b = true;
  while (Wire.available()) { // slave may send less than requested
    b = Wire.read(); // receive a byte as character
  }
  delay(10);
  Serial.print("Is open?");
  Serial.println(b);
  if (!b) {
    Wire.beginTransmission(ADDR); // transmit to device #8
    Wire.write("opened");        // sends five bytes
    Wire.endTransmission();    // stop transmitting
    delay(10);
    Serial.println("Opened");

    Wire.requestFrom(ADDR, 1);    // request 1 byte from slave device #8
    while (Wire.available()) { // slave may send less than requested
      b = Wire.read(); // receive a byte as character
    }
    delay(10);

    Serial.print("Is open?");
    Serial.println(b);
    if (b) {
      Serial.println("Green");
      blink(LED_G_PIN);
      return true;
    }
    else {
      Serial.println("Red");
      blink(LED_R_PIN);
    }
  } else {
    Serial.println("Yellow");
    blink(LED_Y_PIN);
  }
  return false;

}

void blink(int ledPin) {
  digitalWrite(ledPin, HIGH);
  delay(1000);
  digitalWrite(ledPin, LOW);
}

